@@ -5,14 +5,15 @@
 
 #include "../MaraDns.h"
 
-/* Determine the length of a domain-name label
+/* Determine the length of a domain-name label (if it is a compressed
+   label, determine the length of the compressed portion)
    Input: the js_string obejct with the domain label in question,
           the offset from the beginning of the js_string object
           with the domain label
    Output: The length of the label, JS_ERROR on error
 */
 
-int dlabel_length(js_string *raw, int offset) {
+int dlabel_length(js_string *raw, unsigned int offset) {
 
     int length;
     unsigned char toread;
@@ -28,6 +29,12 @@ int dlabel_length(js_string *raw, int offset) {
         return JS_ERROR;
     toread = *(raw->string + offset);
     while(length < 256 && toread > 0) {
+        if(toread >= 192) /* compressed label */ {
+            if(length + 2 + offset <= raw->unit_count)
+	       return length + 2;
+            else
+	       return JS_ERROR;
+            }
         if(toread > 63)
             return JS_ERROR; /* No EDNS support */
         length += toread + 1;
@@ -44,6 +51,25 @@ int dlabel_length(js_string *raw, int offset) {
 
     }
 
+/* Given a q_header structure, initialize the values of the structure
+   to sane values */
+
+void init_header(q_header *header) {
+    header->id = 0;
+    header->qr = 0;
+    header->opcode = 0;
+    header->aa = 0;
+    header->tc = 0;
+    header->rd = 0;
+    header->ra = 0;
+    header->z  = 0;
+    header->rcode = 0;
+    header->qdcount = 0;
+    header->ancount = 0;
+    header->nscount = 0;
+    header->arcount = 0;
+    }
+
 /* Given a q_header structure, and a js object to put the raw UDP data
    in, convert a q_header structure to raw UDP data. 
    input: pointer to q_header structure, pointer to js_string object
@@ -207,6 +233,7 @@ int make_question(q_question *question, js_string *js) {
         }
 
     /* Since counter is now sitting on the dot after .com, increment it */
+    *(js->string + offset + counter) = 0;
     counter++;
 
     /* Add the QTYPE and QCLASS to the raw UDP data */
@@ -280,6 +307,7 @@ int read_question(js_string *js,q_question *question, int offset) {
         }
 
     /* Since counter is now sitting on the dot after .com, increment it */
+    *(question->qname->string + counter) = 0;
     counter++;
 
     /* Get the QTYPE and QCLASS from the raw UDP data */
@@ -550,7 +578,7 @@ int email_2rfc1035(js_string *hostname) {
      int counter,seen;
      unsigned char *raw, *towrite;
 
-     int firstat = 1;
+     int firstat;
 
      /* Sanity checks */
      if(js_has_sanity(hostname) == JS_ERROR)
@@ -563,8 +591,26 @@ int email_2rfc1035(js_string *hostname) {
      /* Store the length of the first part of the hostname */
      raw = towrite = hostname->string;
 
+     /* If there are no @s in name, but there are dots, make first dot
+        an at.  Too many people are used to BIND's buggy handling of
+	email addresses */
+     seen = 0;
+     firstat = 0;
+     for(counter = 0;counter < hostname->unit_count;counter++) {
+         if(*(hostname->string + counter) == '@' && firstat == 0) {
+	     firstat = counter;
+	     }
+         if(*(hostname->string + counter) == '.' && seen == 0) {
+	     seen = counter;
+	     }
+         }
+     if(seen > 0 && firstat == 0) {
+         *(hostname->string + seen) = '@';
+	 }
+
      /* Convert the at/dots in to hostname delimiters */
      seen = counter = 0;
+     firstat = 1;
      while(counter < 256 && counter < hostname->unit_count) {
          counter++;
          if(counter > hostname->unit_count) 
@@ -724,6 +770,7 @@ int read_soa(js_string *js, rr_soa *soa, int offset) {
          
         }
 
+    *(soa->mname->string + counter) = 0;
     counter++; /* Do not forget the dot after .com */
     offset += counter;
     if(soa->mname->max_count < counter)
@@ -755,6 +802,7 @@ int read_soa(js_string *js, rr_soa *soa, int offset) {
          
         }
 
+    *(soa->rname->string + counter) = 0;
     counter++; /* Do not forget the dot after .com */
     if(soa->rname->max_count < counter)
         return JS_ERROR;
@@ -822,6 +870,7 @@ int read_ns(js_string *in, js_string *out, int offset) {
          
         }
 
+    *(out->string + counter) = 0;
     counter++; /* Do not forget the dot after .com */
     offset += counter;
     if(out->max_count < counter)
