@@ -39,7 +39,7 @@ js_string *js_create(unsigned int max_count, unsigned int unit_size) {
     return new;
     }
 
-#ifdef DEBUG_MEMORY
+#ifdef DEBUG
 /* js_create_DEBUG: Create a new string object, keeping track of the
                     memory we allocate
    input: The maximum unit count and unit size of the new string object,
@@ -61,14 +61,15 @@ js_string *js_create_DEBUG(unsigned int max_count, unsigned int unit_size,
     new->encoding = JS_BINARY;
     /* Allocate memory for character string, return on error */
     /* The 3 is a security margin */
-    if((new->string = js_alloc(max_count + 3,unit_size)) == (void *)0) {
+    if((new->string = js_alloc_DEBUG(max_count + 3,unit_size,whence)) 
+       == (void *)0) {
         js_dealloc(new);
         return (js_string *)0;
         }
 
     return new;
     }
-#else  /* DEBUG_MEMORY */
+#else  /* DEBUG */
 #define js_create_DEBUG(x,y,z) js_create(x,y)
 #endif /* DEBUG */
 
@@ -849,8 +850,8 @@ int js_qprepend(char *toprepend, js_string *target) {
 
     /* Allocate memory for the appending string if necessary */
     if(temp == 0)
-        /* We really should make 1024 here
-           strlen(toappend) + js_octets(target)
+        /* We really should make 1024 here */
+        /* strlen(toappend) + js_octets(target)
            That would necessitate moving this to JsStrOS.c */
         if((temp = js_create(1024,target->unit_size)) == 0)
             return JS_ERROR;
