@@ -169,7 +169,7 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
     /* OK, copy over the data that extends from the beginning of the
        line to the first pipe */
     if(js_substr(line,name,0,pipep) == JS_ERROR)
-        return JS_ERROR;
+	goto clean;
     /* Try to make that string a RFC1035 domain query */
     tempp = hname_2rfc1035(name);
     if(tempp == JS_ERROR) {
@@ -313,7 +313,7 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
             pipep=tempp;
             /* Get the rest of the line */
             tempp = js_length(line) - pipep - 2;
-            if(tempp < 0 || tempp > 255)
+            if(tempp < 0 || tempp > (MAX_RECORD_LENGTH - 1))
                 goto clean;
             if(js_substr(line,data,pipep + 1,tempp) /* Chop off the final \n */
                == JS_ERROR)
