@@ -15,58 +15,66 @@ extern int js_space_chars();
 
 /* Keywords that are non-dictionary strings in Mara's rc file */
 
-#define KEYCOUNT 24
+#define KEYCOUNT 27
 
 char *keywords[KEYCOUNT] = {
         "bind_address", /* Address to bind to (0.0.0.0 is all addresses) */
         "chroot_dir",   /* Directory to chroot to */
-        "maradns_uid",  /* UID that MaraDNS will run as */
-        "maradns_gid",  /* GID that MaraDNS will run as (optional) */
-        "maxprocs",     /* Maximum number of udp threads or tcp processes */
-        "max_tcp_procs", /* Maximum number of tcp processes */
-        "verbose_level", /* How verbose error messages are */
-        "zone_transfer_acl", /* ACL of IPs allowed to perform zone transfers */
-        "recursive_acl", /* ACL of IPs allowed to perform recursive queries */
-        "tuzona_inetd", /* Do we run the zoneserver from inetd (security risk:
-                           enabling this disables the zone_transfer_acl) */
-        "no_fingerprint", /* Do we remove certain distinctive traits that
-                             MaraDNS has; e.g. a TXT query of
-                             erre-con-erre-cigarro.maradns.org. tells us
-                             the version number of the MaraDNS server */
+	"debug_response_delay", /* Seconds to delay in sending a reply
+	                           (useful for debugging purposes) */
+        "debug_msg_level", /* The level of debug messages to allow.
+	                      0: None; 1: version number with 
+			      Terre-con-erre-cigarro.maradns.org.
+			      2: 1 & number of threads currently running
+			        with Tnumthreads.maradns.org. */
         "default_rrany_set", /* What kind of records do we return when someone
                                 sends us an ANY query.  Note:  This only
                                 changes MaraDNS' behavior as an authoritative
                                 nameserver */
-        "timeout_seconds", /* The amount of time we wait for a foreign
-                              DNS server to respond before we give up and
-                              try the next DNS server. */
+        "hide_disclaimer", /* Whether to hide the disclaimer */
+        "maradns_uid",  /* UID that MaraDNS will run as */
+        "maradns_gid",  /* GID that MaraDNS will run as (optional) */
         "max_ar_chain", /* Maximum number of records in a chain of
                            records in the AR section.  Note:  This has
                            to be one or round-robin rotates are disabled */
         "max_chain", /* Maximum number of records in a chain of records for
                         a given host name */
-        "max_total", /* Maximum number of records total */
-        "random_seed_file", /* File with a seed for the random number
-                               generator.  Note:  This has to be truly
-                               random for it to be a secure random
-                               number generator.  */
-        "spammers", /* List of IPs of spam-friendly DNS servers.  MaraDNS will
-                       refuse to query any DNS server on this list */
-        "hide_disclaimer", /* Whether to hide the disclaimer */
         "max_glueless_level", /* Maximum glueless level allowed when
                                  performing recursive lookups.  The
                                  default value is 6 */
         "max_queries_total", /* Maximum total number of queries allowed
                                 when chasing down a host name.  Default
                                 is 32 */
+        "max_tcp_procs", /* Maximum number of tcp processes */
+        "max_total", /* Maximum number of records total */
         "maximum_cache_elements", /* Maximum number of elements in the cache */
-	"debug_response_delay", /* Seconds to delay in sending a reply
-	                           (useful for debugging purposes) */
-        "debug_msg_level" /* The level of debug messages to allow.
-	                     0: None; 1: version number with 
-			     Terre-con-erre-cigarro.maradns.org.
-			     2: 1 & number of threads currently running
-			        with Tnumthreads.maradns.org. */
+        "maxprocs",     /* Maximum number of udp threads or tcp processes */
+        "min_ttl_cname", /* Minimum TTL for CNAME records */
+	"min_ttl", /* Minimum TTL for records */
+        "no_fingerprint", /* Do we remove certain distinctive traits that
+                             MaraDNS has; e.g. a TXT query of
+                             erre-con-erre-cigarro.maradns.org. tells us
+                             the version number of the MaraDNS server */
+        "random_seed_file", /* File with a seed for the random number
+                               generator.  Note:  This has to be truly
+                               random for it to be a secure random
+                               number generator.  */
+        "recursive_acl", /* ACL of IPs allowed to perform recursive queries */
+        "spammers", /* List of IPs of spam-friendly DNS servers.  MaraDNS will
+                       refuse to query any DNS server on this list */
+        "timeout_seconds", /* The amount of time we wait for a foreign
+                              DNS server to respond before we give up and
+                              try the next DNS server. */
+        "timestamp_type", /* People get religious about what is
+	                     the True Timestamp Format (TM); as a
+		     	     result we need to allow the user to 
+			     choose this or the mailing list will
+			     have a heated discussion about what 
+			     format to use every month or so */
+        "tuzona_inetd", /* Do we run the zoneserver from inetd (security risk:
+                           enabling this disables the zone_transfer_acl) */
+        "verbose_level", /* How verbose error messages are */
+        "zone_transfer_acl" /* ACL of IPs allowed to perform zone transfers */
         };
 
 /* mara_goodjs: Determine if a given js_string object is a valid string
@@ -220,11 +228,12 @@ int write_kvar(js_string *name, js_string *value) {
 
 /* Keywords that are dictionaries in the MaraDNS rc file */
 
-#define DKEYCOUNT 3
+#define DKEYCOUNT 4
 
 char *dkeywords[DKEYCOUNT] = {
         "csv1",
         "root_servers",
+	"upstream_servers",
         "ipv4_alias" };
 
 mhash *dvar[DKEYCOUNT];
@@ -351,6 +360,10 @@ int read_dvar(js_string *name, js_string *key, js_string *value) {
 
     /* Find out where the actual value is */
     place = mhash_get_js(dvar[num],key);
+    /* This is a little klunky; there really needs to be a special
+       "yes, the hash is there, but I can't find the element you are 
+        seeking" error code.  However, other MaraDNS code depends on
+	this buggy behavior, so I am not changing this quite yet */
     if(place == 0)
         return JS_ERROR;
 
@@ -880,7 +893,7 @@ int read_mararc(js_string *fileloc,js_string *errorstr,int *errorret) {
         return JS_SUCCESS;
         }
     while(!js_buf_eof(file)) {
-        if((js_buf_getline(file,line)) == JS_ERROR) {
+        if((js_buf_getline(file,line)) <= JS_ERROR) {
             js_qstr2js(errorstr,L_JSBUFGETLINE_FATAL); /* "Fatal error calling js_buf_getline" */
             js_close(file);
             return JS_ERROR;
@@ -956,6 +969,8 @@ int read_mararc(js_string *fileloc,js_string *errorstr,int *errorret) {
     if(!error)
         *errorret = 0;
 
+    js_close(file);
+
     return JS_SUCCESS;
     }
 
