@@ -37,13 +37,15 @@ main() {
         harderror("Unable to read example.uncompressed contents");
     js_close(&desc);
 
-    js_show_stdout(indata);
+    show_esc_stdout(indata);
 
     /* OK, try to compress the data */
 
-    printf("\n\n%d\n",compress_data(indata,outdata));
+    printf("\n\n%d\n\n",compress_data(indata,outdata));
 
-    js_show_stdout(outdata);
+    show_esc_stdout(outdata);
+
+    printf("\n");
 
     }
 
