@@ -28,6 +28,34 @@ int verbose_mode = 0; /* Whether to have short or verbose output;
                          1: verbose output; any other value: short output */
 int timeout = 21;
 
+/* So -Wall doesn't gag */
+extern int decomp_init();
+
+/* Octal escape any non-ASCII sequences in the askmara output; almost idential
+   to show_esc_stdout */
+int escape_stdout(js_string *js) {
+    int counter = 0;
+    unsigned char this;
+
+    /* Sanity checks */
+    if(js_has_sanity(js) < 0)
+        return -1;
+    if(js->unit_size != 1)
+        return -1;
+
+    while(counter < js->unit_count) {
+        this = *(js->string + counter);
+	if(this < 32 || this > 126) 
+	    printf("\\%03o",this);
+        else if(this == '\\') 
+	    printf("\\%c",this);
+        else
+	    printf("%c",this);
+        counter++;
+        }
+    return 1;
+    }
+
 /* Generate a psudo-random query-id based on the hostname we give it.
    This helps us test the server against a large number of query IDs */
 
@@ -39,7 +67,8 @@ u_int16_t gen_id(char *hostname) {
     keyInstance r_keyInst;
     cipherInstance r_cipherInst;
     unsigned char crypto_key[34];
-    int desc, len;
+    int desc, len, counter;
+    char *mc1, *mc2;
     /* END RNG USING CODE */
     int ret = 4321;
 
@@ -49,15 +78,31 @@ u_int16_t gen_id(char *hostname) {
 
     /* Initialize the keys, including the "binKey" (is this used?) */
     memset(r_binKey,'0',16);
-    memset(crypto_key,'',16);
+    mc1 = "ñ";
+    mc2 = mc1;
+    /* Set the crypto key to 'ññññññññññññññññ' (UTF-8 encoding) */
+    for(counter = 0; counter < 16; counter++) {
+        *(crypto_key + counter) = *mc2;
+	mc2++;
+	if(*mc2 == 0) 
+	   mc2 = mc1;
+        }
+    /*memset(crypto_key,'ñ',16);*/
     /* Make the first 16 characters of the hostname the key (padded by the
-        character--do you see a n~ there?) */
+       ñ character--do you see a n~ there?) */
     strncpy(crypto_key,hostname,16);
-    /* Set the plaintext block to '' */
-    memset(r_inBlock,'',16);
+    /* Set the plaintext block to 'ññññññññññññññññ' (UTF-8 encoding) */
+    mc2 = mc1;
+    for(counter = 0; counter < 16; counter++) {
+        *(r_inBlock + counter) = *mc2;
+	mc2++;
+	if(*mc2 == 0) 
+	   mc2 = mc1;
+        }
+    /*memset(r_inBlock,'ñ',16);*/
     /* If the hostname is longer than 16 characters, make the plaintext
        the longer part of the hostname */
-    if(strlen(hostname) > 16)
+    if(js_strnlen(hostname,1000000) > 16)
     strncpy(r_inBlock,hostname+16,16);
 
     if(makeKey(&r_keyInst, DIR_ENCRYPT, 128, crypto_key) != 1)
@@ -96,6 +141,8 @@ main(int argc, char **argv) {
     int n; /* Select() return value */
     int nrd = 0; /* whether recursion is desired or not 1: No recursion;
                    0: recursion desired */
+    int id;
+    int attempts = 10;
 
     /* Determine what the query string is */
     if(argc < 2 || argc > 6)
@@ -151,9 +198,10 @@ main(int argc, char **argv) {
 
     len_inet = sizeof(dns_udp);
 
+    id = gen_id(argv[1 + verbose_mode]);
     /* Format a DNS request */
-    /* DNS header -> using dig defaults */
-    header.id = gen_id(argv[1 + verbose_mode]);
+    /* DNS header */
+    header.id = id;
     header.qr = 0;
     header.opcode = 0;
     header.aa = 0;
@@ -191,34 +239,49 @@ main(int argc, char **argv) {
     make_hdr(&header,outdata);
     make_question(&question,outdata);
 
-    /* Send out a DNS request */
-    if(sendto(s,outdata->string,outdata->unit_count,0,
-       (struct sockaddr *)&dns_udp,len_inet) < 0)
-        harderror(L_UDP_NOSEND); /* Unable to send UDP packet */
+    do {
+        /* Send out a DNS request */
+        if(sendto(s,outdata->string,outdata->unit_count,0,
+          (struct sockaddr *)&dns_udp,len_inet) < 0)
+           harderror(L_UDP_NOSEND); /* Unable to send UDP packet */
 
-    /* Wait for a reply from the DNS server */
-    FD_ZERO(&rx_set);
-    FD_SET(s,&rx_set);
-    maxd = s + 1;
-    tv.tv_sec = timeout;
-    tv.tv_usec = 0;
-    n = select(maxd,&rx_set,NULL,NULL,&tv);
-    if(n == -1)  /* select error */
-        harderror("Select() failed");
-    if(n == 0) /* Timeout */
-        harderror("Timeout");
-    if((count = recvfrom(s,indata->string,indata->max_count,0,
+        /* Wait for a reply from the DNS server */
+        FD_ZERO(&rx_set);
+        FD_SET(s,&rx_set);
+        maxd = s + 1;
+        tv.tv_sec = timeout;
+        tv.tv_usec = 0;
+        n = select(maxd,&rx_set,NULL,NULL,&tv);
+        if(n == -1)  /* select error */
+            harderror("Select() failed");
+        if(n == 0) /* Timeout */
+            harderror("Timeout");
+        if((count = recvfrom(s,indata->string,indata->max_count,0,
                         (struct sockaddr *)&dns_udp,&len_inet)) < 0)
-        harderror(L_DNS_R_ERROR); /* Problem getting DNS server response */
+            harderror(L_DNS_R_ERROR); /* Problem getting DNS server response */
 
-    indata->unit_count = count;
+        indata->unit_count = count;
 
+        /* Get the ID from the remote response */
+        if(count > 2) {
+	   id = (*(indata->string) << 8) | *(indata->string + 1);
+	   }
+        else {
+	   id = header.id + 1;
+	   }
+
+        attempts--;
+
+	/* If the id doesn't match, get a reply again */
+	} while(id != header.id && attempts > 0); 
+
+    decomp_init(0);
     decompress_data(indata,uindata);
 
     /*
-    js_show_stdout(indata);
-    printf("%s",L_NEWLINE);
-    js_show_stdout(uindata);
+    escape_stdout(indata);
+    printf("%s%s",L_NEWLINE,L_NEWLINE);
+    escape_stdout(uindata);
     */
 
     if(verbose_mode == 1)
@@ -272,7 +335,7 @@ int verbose_output(js_string *uindata) {
         place += count;
         printf("%s",L_QNAME); /* Question name */
         hname_translate(question.qname,question.qtype);
-        js_show_stdout(question.qname);
+        escape_stdout(question.qname);
         printf("%s%s%d%s",L_NEWLINE,L_QTYPE,question.qtype,L_NEWLINE);
         printf("%s%d%s",L_QCLASS,question.qclass,L_NEWLINE); /* Ques. Class */
         }
@@ -327,7 +390,7 @@ int out_answer(js_string *uindata,int *place) {
         *place += count;
         hname_translate(rr_hdr.name,rr_hdr.type);
         printf("%s",L_RNAME); /* Record name */
-        js_show_stdout(rr_hdr.name);
+        escape_stdout(rr_hdr.name);
         printf("\nRecord type: %d\n",rr_hdr.type);
         printf("Record class: %d\n",rr_hdr.class);
         printf("Record TTL: %u\n",rr_hdr.ttl);
@@ -339,9 +402,9 @@ int out_answer(js_string *uindata,int *place) {
             hname_translate(soa.mname,RR_SOA);
             hname_translate(soa.rname,RR_MX);
             printf("%s",L_S_MNAME); /* SOA mname */
-            js_show_stdout(soa.mname);
+            escape_stdout(soa.mname);
             printf("%s%s",L_NEWLINE,L_S_RNAME); /* SOA rname */
-            js_show_stdout(soa.rname);
+            escape_stdout(soa.rname);
             printf("%s%s%u%s",L_NEWLINE,L_S_SERIAL,soa.serial,L_NEWLINE);
             printf("%s%d%s",L_S_REFRESH,soa.refresh,L_NEWLINE);
             printf("%s%d%s",L_S_RETRY,soa.retry,L_NEWLINE);
@@ -357,7 +420,7 @@ int out_answer(js_string *uindata,int *place) {
             printf("%s%d%s",L_M_PREF,mx.preference,L_NEWLINE); /* MX pref. */
             printf("%s",L_M_EX); /* MX exchange */
             hname_translate(mx.exchange,RR_MX);
-            js_show_stdout(mx.exchange);
+            escape_stdout(mx.exchange);
             printf("%s%s",L_NEWLINE,L_NEWLINE);
             }
         else if(rr_hdr.type == RR_NS || rr_hdr.type == RR_CNAME ||
@@ -365,7 +428,7 @@ int out_answer(js_string *uindata,int *place) {
             printf("%s",L_DATA); /* Data: */
             read_ns(uindata,mx.exchange,*place);
             hname_translate(mx.exchange,rr_hdr.type);
-            js_show_stdout(mx.exchange);
+            escape_stdout(mx.exchange);
             printf("%s%s",L_NEWLINE,L_NEWLINE);
             }
         else if(rr_hdr.type == RR_A) {
@@ -380,7 +443,7 @@ int out_answer(js_string *uindata,int *place) {
         else if(rr_hdr.type = RR_TXT) {
             printf("%s",L_TXT); /* Text String */
             read_txt(uindata,mx.exchange,*place);
-            js_show_stdout(mx.exchange);
+            escape_stdout(mx.exchange);
             printf("%s%s",L_NEWLINE,L_NEWLINE);
             }
         else
@@ -416,6 +479,11 @@ int csv1_compatible_output(js_string *uindata, js_string *query) {
         count = read_question(uindata,&question,place);
         if(count < 0) /* Error handling */
             harderror(L_QERROR); /* Error reading question */
+        printf("# Question: ");
+	if(hname_translate(question.qname,question.qtype) == JS_ERROR) 
+	    harderror(L_QERROR);
+        show_esc_stdout(question.qname);
+	printf("\n");
         place += count;
         }
 
@@ -452,6 +520,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     rr_soa soa;
     rr_mx mx;
     int count;
+    js_string *lower1, *lower2;
 
     /* Create the needed strings */
     if((rr_hdr.name = js_create(512,1)) == 0)
@@ -462,6 +531,10 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
        harderror(L_C_RNAME); /* js_create with soa.rname in oa */
     if((mx.exchange = js_create(512,1)) == 0)
        harderror(L_C_MXEXC); /* js_create with mx.exchange in oa */
+    if((lower1 = js_create(512,1)) == 0)
+       harderror(L_C_MXEXC); /* js_create with mx.exchange in oa */
+    if((lower2 = js_create(512,1)) == 0)
+       harderror(L_C_MXEXC); /* js_create with mx.exchange in oa */
 
     /* Start printing out the data header */
     count = read_rr_h(uindata,&rr_hdr,*place);
@@ -470,12 +543,18 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     *place += count;
 
     /* Display a hash if the record name is not the same as the
-       query */
-    if(js_issame(query,rr_hdr.name) != 1)
+       query (case_insensitive matching) */
+    js_copy(query,lower1);
+    js_copy(rr_hdr.name,lower2);
+    lower1->encoding = JS_US_ASCII;
+    lower2->encoding = JS_US_ASCII;
+    js_tolower(lower1);
+    js_tolower(lower2);
+    if(js_issame(lower1,lower2) != 1)
         printf("#");
 
     hname_translate(rr_hdr.name,rr_hdr.type);
-    js_show_stdout(rr_hdr.name);
+    escape_stdout(rr_hdr.name);
     /* We also show the type if this is an unsupported record type */
     if(rr_hdr.type != RR_A &&
        rr_hdr.type != RR_NS &&
@@ -492,9 +571,9 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
         hname_translate(soa.mname,RR_SOA);
         hname_translate(soa.rname,RR_MX);
         printf("|",L_S_MNAME); /* SOA mname */
-        js_show_stdout(soa.mname);
+        escape_stdout(soa.mname);
         printf("|",L_NEWLINE,L_S_RNAME); /* SOA rname */
-        js_show_stdout(soa.rname);
+        escape_stdout(soa.rname);
         printf("|%u",soa.serial,L_NEWLINE);
         printf("|%d",soa.refresh,L_NEWLINE);
         printf("|%d",soa.retry,L_NEWLINE);
@@ -509,14 +588,14 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
         read_ns(uindata,mx.exchange,*place + 2);
         printf("|%d",mx.preference,L_NEWLINE); /* MX pref. */
         hname_translate(mx.exchange,-2); /* Makes the char a pipe */
-        js_show_stdout(mx.exchange);
+        escape_stdout(mx.exchange);
         printf("\n");
         }
     else if(rr_hdr.type == RR_NS || rr_hdr.type == RR_CNAME ||
             rr_hdr.type == RR_PTR) {
         read_ns(uindata,mx.exchange,*place);
         hname_translate(mx.exchange,-2); /* First char is a pipe */
-        js_show_stdout(mx.exchange);
+        escape_stdout(mx.exchange);
         printf("\n");
         }
     else if(rr_hdr.type == RR_A) {
@@ -531,7 +610,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     else if(rr_hdr.type = RR_TXT) {
         printf("|"); 
         read_txt(uindata,mx.exchange,*place);
-        js_show_stdout(mx.exchange);
+        escape_stdout(mx.exchange);
         printf("\n");
         }
     else {
@@ -541,3 +620,4 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     *place += rr_hdr.rdlength; /* To do: read the RD data itself for
                                   all MaraDNS supported RRs */
     }
+
