@@ -30,6 +30,7 @@
 /* Function prototypes */
 #include "../dns/functions_dns.h"
 #include "../parse/functions_parse.h"
+extern int decomp_init();
 
 /* Our one global variable: The number of children processes running */
 int num_children = 0;
@@ -164,11 +165,11 @@ int serve_zone(int connect) {
     uint32 ttl;
 
     /* Create the js_string objects */
-    if((name = js_create(384,1)) == 0)
+    if((name = js_create((MAX_RECORD_LENGTH * 2) + 3,1)) == 0)
         return JS_ERROR;
     if(js_set_encode(name,MARA_LOCALE) == JS_ERROR)
         return JS_ERROR;
-    if((data = js_create(384,1)) == 0) {
+    if((data = js_create((MAX_RECORD_LENGTH * 2) + 3,1)) == 0) {
         js_destroy(name);
         return JS_ERROR;
 	}
@@ -338,7 +339,7 @@ int serve_zone(int connect) {
         /* Start reading the zone file */
 	while(!js_buf_eof(&desc)) {
             /* Get the line */
-            if(js_buf_getline(&desc,data) == JS_ERROR)
+            if(js_buf_getline(&desc,data) <= JS_ERROR)
                 goto clean_ndzqrs;
             /* Process the % character and any \ escape sequences */
             if(bs_process(data,name,zone) == JS_ERROR)
@@ -430,7 +431,7 @@ int serve_zone(int connect) {
     /* Read all of the lines from the zone file one by one */
     while(!js_buf_eof(&desc)) {
         /* Get the line */
-	if(js_buf_getline(&desc,data) == JS_ERROR) 
+	if(js_buf_getline(&desc,data) <= JS_ERROR) 
 	    goto clean_ndzqrs;
         /* Process the % character and any \ escape sequences */
         if(bs_process(data,name,zone) == JS_ERROR) 
@@ -630,7 +631,7 @@ on %s\nFor usage information, type in 'man maradns'" */
 #ifndef DARWIN
     rlim.rlim_cur = rlim.rlim_max = maxprocs;
 
-#ifndef SOLARIS
+#ifdef RLIMIT_NPROC
     if(setrlimit(RLIMIT_NPROC,&rlim) != 0)
         harderror(L_SETMAX); /* "Unable to set maximum number of processes" */
 #endif /* SOLARIS */
@@ -720,6 +721,9 @@ on %s\nFor usage information, type in 'man maradns'" */
             harderror(L_ACL_LIST); /* "Could not make ip ACL list" */
         }
 
+    /* Initialize decompression */
+    decomp_init(0);
+
     /* If we are being called from an external helper, such as inetd
        or tcpserver */
     if(inetd == 1) {
